﻿using System;
using Microsoft.SqlServer.Dts.Pipeline.Wrapper;
using Microsoft.SqlServer.Dts.Runtime;
using wrap = Microsoft.SqlServer.Dts.Runtime.Wrapper;


namespace CreateAndRunPackage
{
    class Program
    {
        static void Main(string[] args)
        {
            CreateAndRunSamplePackage();
        }

        /// <summary>
        /// Create a sample package, run it and print the execution result
        /// </summary>
        private static void CreateAndRunSamplePackage()
        {
            using (Project project = Project.CreateProject())
            {
                Package package = CreateSamplePackage();
                project.PackageItems.Add(package, "package1.dtsx");

                //run the package
                DTSExecResult result = project.PackageItems[0].Package.Execute(null, null, null, null, null);
                Console.WriteLine("Execution result: " + result);
            }
        }

        /// <summary>
        /// Create a sample package with a data flow
        /// </summary>
        private static Package CreateSamplePackage()
        {
            const string OututFileName = @"d:\OutputData.txt";
            const string OleDBConnectionString = @"Provider=SQLNCLI11;Integrated Security=SSPI;Persist Security Info=False;Initial Catalog=ssisdb;Data Source=(local);Auto Translate=False;";
            const string ViewName = @"[catalog].[catalog_properties]";

            Package package = new Package();

            //add connection managers
            ConnectionManager oleDBConnection = AddOleDBConnectionManager(package, OleDBConnectionString);
            ConnectionManager fileConnection = AddFlatFileConnectionManager(package, OututFileName);

            //add data flow task
            TaskHost th = package.Executables.Add("STOCK:PipelineTask") as TaskHost;
            th.Name = "DataFlow";
            MainPipe dataFlow = th.InnerObject as MainPipe;

            //add data flow components
            IDTSComponentMetaData100 oledbSource = AddOLEDBSource(package, dataFlow, oleDBConnection, ViewName);
            IDTSComponentMetaData100 flatfileDestination =
                AddFlatFileDestination(package, dataFlow, oledbSource.OutputCollection[0], fileConnection);

            return package;
        }

        /// <summary>
        /// Add an OLE DB connection Manager into the package
        /// </summary>
        private static ConnectionManager AddOleDBConnectionManager(Package package, string connectionString)
        {
            ConnectionManager oleDBConnection = package.Connections.Add("OLEDB");

            oleDBConnection.Name = "OLEDBConnection";
            oleDBConnection.ConnectionString = connectionString;

            return oleDBConnection;
        }

        /// <summary>
        /// Add a Flat file connection manager into the package
        /// </summary>
        private static ConnectionManager AddFlatFileConnectionManager(Package package, string fileName)
        {
            ConnectionManager fileConnection = package.Connections.Add("FLATFILE");

            fileConnection.Properties["ConnectionString"].SetValue(fileConnection, fileName);
            fileConnection.Properties["Format"].SetValue(fileConnection, "Delimited");
            fileConnection.Properties["DataRowsToSkip"].SetValue(fileConnection, 0);
            fileConnection.Properties["ColumnNamesInFirstDataRow"].SetValue(fileConnection, false);
            fileConnection.Properties["Name"].SetValue(fileConnection, "FlatFileConnection");
            fileConnection.Properties["RowDelimiter"].SetValue(fileConnection, "\r\n");
            fileConnection.Properties["TextQualifier"].SetValue(fileConnection, "\"");

            return fileConnection;
        }

        /// <summary>
        /// Create an OLE DB Source which reads the given table or view
        /// </summary>
        private static IDTSComponentMetaData100 AddOLEDBSource(Package package, MainPipe dataFlow,
            ConnectionManager oleDBConnection, string tableOrViewName)
        {
            //create and initialize the component
            IDTSComponentMetaData100 oledbSource = dataFlow.ComponentMetaDataCollection.New();
            oledbSource.ComponentClassID = "DTSAdapter.OLEDBSource";
            oledbSource.Name = "OLEDBSource";

            CManagedComponentWrapper componentWrapper = oledbSource.Instantiate();
            componentWrapper.ProvideComponentProperties();

            //bind the connection manager
            oledbSource.RuntimeConnectionCollection[0].ConnectionManagerID = oleDBConnection.ID;
            oledbSource.RuntimeConnectionCollection[0].ConnectionManager
                = DtsConvert.GetExtendedInterface(oleDBConnection);

            //set custom component properties
            componentWrapper.SetComponentProperty("OpenRowset", tableOrViewName);
            componentWrapper.SetComponentProperty("AccessMode", 0);

            //acquire Connections and reinitialize the component
            componentWrapper.AcquireConnections(null);
            componentWrapper.ReinitializeMetaData();
            componentWrapper.ReleaseConnections();

            return oledbSource;
        }

        /// <summary>
        /// Add a flat file destination, and connects it to the given source output
        /// </summary>
        private static IDTSComponentMetaData100 AddFlatFileDestination(Package package, MainPipe dataFlow,
            IDTSOutput100 sourceOutput, ConnectionManager fileConnection)
        {
            // Add the component to the dataFlow metadata collection
            IDTSComponentMetaData100 flatfileDestination = dataFlow.ComponentMetaDataCollection.New();
            flatfileDestination.ComponentClassID = "DTSAdapter.FlatFileDestination";
            flatfileDestination.Name = "FlatFileDestination";

            CManagedComponentWrapper componentWrapper = flatfileDestination.Instantiate();
            componentWrapper.ProvideComponentProperties();

            //bind the connection manager
            flatfileDestination.RuntimeConnectionCollection[0].ConnectionManagerID
                = fileConnection.ID;
            flatfileDestination.RuntimeConnectionCollection[0].ConnectionManager
                = DtsConvert.GetExtendedInterface(fileConnection);

            //connect the component to source component
            dataFlow.PathCollection.New().AttachPathAndPropagateNotifications(sourceOutput,
                flatfileDestination.InputCollection[0]);

            // Add columns to the FlatFileConnectionManager
            AddColumnsToConnectionManager(package, flatfileDestination, fileConnection);

            // Acquire the connection, reinitialize the metadata, 
            // map the columns, then release the connection.
            componentWrapper.AcquireConnections(null);
            componentWrapper.ReinitializeMetaData();
            MapFlatFileDestinationColumns(flatfileDestination, componentWrapper);
            componentWrapper.ReleaseConnections();

            return flatfileDestination;
        }

        /// <summary>
        /// Add all input columns in the destination component to the assocaited connection manager
        /// </summary>
        private static void AddColumnsToConnectionManager(Package package, IDTSComponentMetaData100 fileDestination,
            ConnectionManager fileConnection)
        {
            wrap.IDTSConnectionManagerFlatFile100 ff = fileConnection.InnerObject as wrap.IDTSConnectionManagerFlatFile100;

            IDTSVirtualInputColumnCollection100 virtualInputColumns =
                fileDestination.InputCollection[0].GetVirtualInput().VirtualInputColumnCollection;

            //add all columns with the required format
            for (int columnIndex = 0; columnIndex < virtualInputColumns.Count; columnIndex++)
            {
                wrap.IDTSConnectionManagerFlatFileColumn100 col = ff.Columns.Add();
                if (columnIndex == virtualInputColumns.Count - 1)
                {
                    col.ColumnDelimiter = "\r\n";
                }
                else
                {
                    col.ColumnDelimiter = @",";
                }

                col.ColumnType = "Delimited";
                col.DataType = virtualInputColumns[columnIndex].DataType;
                col.DataPrecision = virtualInputColumns[columnIndex].Precision;
                col.DataScale = virtualInputColumns[columnIndex].Scale;
                wrap.IDTSName100 name = col as wrap.IDTSName100;
                name.Name = virtualInputColumns[columnIndex].Name;
            }
        }

        /// <summary>
        /// Use all input columns and map them to the external columns
        /// </summary>
        private static void MapFlatFileDestinationColumns(IDTSComponentMetaData100 flatfileDestination,
            CManagedComponentWrapper componentWrapper)
        {
            //find and use all input columns
            IDTSVirtualInput100 virtualInput = flatfileDestination.InputCollection[0].GetVirtualInput();
            foreach (IDTSVirtualInputColumn100 column in virtualInput.VirtualInputColumnCollection)
            {
                componentWrapper.SetUsageType(flatfileDestination.InputCollection[0].ID,
                    virtualInput, column.LineageID, DTSUsageType.UT_READONLY);
            }

            //find the corresponding external metadata column.

            foreach (IDTSInputColumn100 inputColumn in flatfileDestination.InputCollection[0].InputColumnCollection)
            {
                IDTSExternalMetadataColumn100 externalColumn =
                    flatfileDestination.InputCollection[0].ExternalMetadataColumnCollection[inputColumn.Name];
                componentWrapper.MapInputColumn(flatfileDestination.InputCollection[0].ID, inputColumn.ID, externalColumn.ID);
            }
        }

    }
}
